using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using WindowsLive.Writer.Api;
using System.Windows.Forms;

// Copyright notice: this file is public domain.
// it contains portions of code from:
// http://blogs.msdn.com/jmstall/archive/2007/01/21/html-clipboard.aspx

namespace FootNote
{

    [WriterPluginAttribute
       ("76D3DA3C-E315-11DB-8A5D-1A9956D89593",
        "Footnote plugin",
        ImagePath = "Images.footnote.png",
        PublisherUrl = "http://paperino.blogsphot.com",
        Description = "Insert a footnote")]


    [InsertableContentSourceAttribute("Footnote")]
    public class Plugin : ContentSource
    {
        const string noteTemplate = "<sup>[{0}]</sup>";
        const string hrefTemplate = "<a href='#fn{0}'><sup>[{0}]</sup></a>";
        const string aNameTemplate = "<hr/><a name='fn{0}'></a><sup>[{0}]</sup>&nbsp;";

        public override DialogResult CreateContent(IWin32Window dialogOwner,

            ref string newContent)
        {

            using (FootNoteForm footnoteForm = new FootNoteForm())
            {

                DialogResult result = footnoteForm.ShowDialog();


                if (result == DialogResult.OK)
                {
                    newContent = PrepareOutput(footnoteForm.noteValueTextbox.Text, footnoteForm.clipboardCheckbox.Checked);

                    if (footnoteForm.clipboardCheckbox.Checked)
                    {
                        CopyToClipboard(PrepareFootNote(footnoteForm.noteValueTextbox.Text, footnoteForm.hyperlinkCheckbox.Checked), null, null);
                    } 
                }
                return result;
            }
        }

        public static string PrepareOutput(string noteValue, bool addToClipboard)
        {
            return String.Format(addToClipboard ? aNameTemplate : noteTemplate, noteValue);
        }

        public static string PrepareFootNote(string htmlString, bool addLink)
        {
            return String.Format(addLink ? hrefTemplate : noteTemplate, htmlString);
        }

        static string To8DigitString(int x)
        {
            return String.Format("{0,8}", x);
        }

    /// <summary>
    /// Clears clipboard and copy a HTML fragment to the clipboard, providing additional meta-information.
    /// </summary>
    /// <param name="htmlFragment">a html fragment</param>
    /// <param name="title">optional title of the HTML document (can be null)</param>
    /// <param name="sourceUrl">optional Source URL of the HTML document, for resolving relative links (can be null)</param>
    public static void CopyToClipboard(string htmlFragment, string title, Uri sourceUrl)
    {
        if (title == null) title = "From Clipboard"; 

        System.Text.StringBuilder sb = new System.Text.StringBuilder();

        // Builds the CF_HTML header. See format specification here:
        // http://msdn.microsoft.com/library/default.asp?url=/workshop/networking/clipboard/htmlclipboard.asp

        // The string contains index references to other spots in the string, so we need placeholders so we can compute the offsets. 
        // The <<<<<<<_ strings are just placeholders. We'll backpatch them actual values afterwards.
        // The string layout (<<<) also ensures that it can't appear in the body of the html because the <
        // character must be escaped.
        string header =
@"Format:HTML Format
Version:1.0
StartHTML:<<<<<<<1
EndHTML:<<<<<<<2
StartFragment:<<<<<<<3
EndFragment:<<<<<<<4
StartSelection:<<<<<<<3
EndSelection:<<<<<<<3
";

        string pre =
@"<!DOCTYPE HTML PUBLIC ""-//W3C//DTD HTML 4.0 Transitional//EN"">
<HTML><HEAD><TITLE>" + title + @"</TITLE></HEAD><BODY><!--StartFragment-->";

        string post = @"<!--EndFragment--></BODY></HTML>";

        sb.Append(header);
        if (sourceUrl != null)
        {
            sb.AppendFormat("SourceURL:{0}", sourceUrl);
        }
        int startHTML = sb.Length;

        sb.Append(pre);
        int fragmentStart = sb.Length;

        sb.Append(htmlFragment);
        int fragmentEnd = sb.Length;

        sb.Append(post);
        int endHTML = sb.Length;

        // Backpatch offsets
        sb.Replace("<<<<<<<1", To8DigitString(startHTML));
        sb.Replace("<<<<<<<2", To8DigitString(endHTML));
        sb.Replace("<<<<<<<3", To8DigitString(fragmentStart));
        sb.Replace("<<<<<<<4", To8DigitString(fragmentEnd));


        // Finally copy to clipboard.
        string data = sb.ToString();
        Clipboard.Clear();
        Clipboard.SetText(data, TextDataFormat.Html);
    }

    }
}
